# a set of functions for postprocessing the output from batwing.
#
#  An example for the shape functions is given at the 
#  end of this file 
#
#  rbat read the output file from batwing and 
#  names all the columns
#
#  The columns then then be accessed by using 
#  outname[,"colname"]  ie out[,"N"] for the 
#  column of population sizes for output out
#  
rbat <-  function(name,throw=1000,partial=F,charinf=F) 
{
##   useage:
##   use out<-rbat(
## 	filename,
##       throw - throws away the first throw rows 
##	partial - do we have partial data?
##	)
  if (charinf==T) {
    o<-read.table(name,header=T);
    } else {
      nam <- scan(name,list(""),n=1);
      l <- length(nam[[1]]);
      o <- matrix(scan(name,skip=1),ncol=l,byrow=T,dimnames=list(NULL,nam[[1]]));
      if (partial) 	o <- o[-nrow(o),];
    }
  if (throw>0) return(o[-c(1:throw),])
  else return(o);
}
#
popsfromshape <- function (shape, name = popnames) 
#   useage: 
#  use popsfromshape(
#  	shape.value - the number of the shape
#       name - a vector of the population names  
#
{
    nam <- name[locations.present(shape, length(name))]
    strnam <- paste(nam, collapse = ",", sep = "")
    paste("(", strnam, ")", sep = "")
}
supported <-function (shapes, howmany = 200) 
#
#  Supported returns the (howmany) most frequent shapes in the 
#  vector or matrix shapes as a list
#  
{
  tab <-  table(as.vector(shapes))
  if (length(tab) < howmany) howmany <- length(tab)
    s <- sort(-tab)
    trees <- names(s[1:howmany])
    freq <- as.vector(-s[1:howmany])
    list(trees = as.integer(trees), freq = freq, p = freq/sum(tab))
  }
#
# a utility function which takes the output of supported and turns 
#  it into a readable table
#
printpopulationtable <-
  function (l, names = popnames, separator = " "
            , line = "\n", digits = 3) 
{
    for (i in c(1:length(l$trees))) {
        topr <- paste(popsfromshape(l$trees[i], names), round(l$p[i], 
            digits), sep = separator)
        cat(topr, line, sep = "");
    }
}
locations.present <-  function(shape,maxlocations=32)
#  locations.present takes a shape output 
# and returns a logical vector with TRUE where a location is
# present and FALSE otherwise 
{
  res <- logical(maxlocations);
  for (i in c(maxlocations:1)) {
    res[i] <- (shape%/%(2^(i - 1))>0);
    shape <- shape%%(2^(i - 1));
  }
  res
}
#  demo
popnames <-c("a","b","c","d","e","f","g","h","i","j")
for (i in seq(1,500,50)) {
  print(paste("the populations in shape",i,"are",
              popsfromshape(i,popnames),"   ",i,"=",
              paste(2^(0:9)[locations.present(i)],collapse=" + ")))
}
trees<-sample(1:1023,4000,replace=T)
printpopulationtable(supported(trees,howmany=10),names=popnames)
#######################################################################
TMRCA <-  function(out,generation.time=20) 
{
    out[,"N"]*generation.time*out[,"T"];
}
#######################################################################
getstats <-  function(o,sf=4,table=T,txt="") {
  if (!is.vector(o)) {
    cls <- ncol(o);
    cat(",mean, median, 95% interval\n");
    for (i in 1:cls)
      getstats(o[,i],sf,table,colnames(o)[i])
    } else {
      if (!table) {
          cat(paste(txt,"\n"));
          cat(paste("mean ",signif(mean(o),digits=sf),"\n"));
          cat(paste("median ",signif(median(o),digits=sf),"\n"));
          cat(
              paste(
                    "95% interval (",
                    paste(
                          signif(
                                 quantile(o,probs=c(0.025,0.975))
                                 ,digits=sf)
                          ,collapse=",")
                    ,")\n"
                    )
              );
	} else {
          if (txt!="") cat(paste(txt,","));
          cat(signif(mean(o),digits=sf),
              signif(median(o),digits=sf),
              paste("(",
		    paste(
                          signif(quantile(o,probs=c(0.025,0.975)),digits=sf)
                          ,sep="",collapse=" "),
		    ")\n",collapse=""),sep=","
              )
	}
    }
}
########################################################################
lposterior <- function(o)
{
  cl <- colnames(o);
  r <- o[,"lltimes"]+o[,"llmut"]+o[,"llprior"];
  if (cl[4]=="llinf") r <- r+o[,"llinf"];
  r
}
########################################################################
trim <- function(o,howmany)
{
  o[-c(1:howmany),];
}
########################################################################
plot.batwing <- function(o)
{
  n<-ncol(o);
  print(n);
  if (n==6) par(mfrow=c(3,2),mar=c(3,2,2,1))
  for (i in 1:n) plot(o[,i],title=colnames(o)[i],type="l") 
  }
########################################################################
ancstate <- function(mat,from,to)
{
  collapse <- function(mat) 
    {
      ## collapse a matrix of factors into a combined factor
      factor(apply(as.matrix(mat),1,paste,collapse=""));
    }
  collapse(mat[,c(from:to)]);
}
########################################################################
